package org.honours.CollectionSpace;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.List;

import org.expeditee.actions.Navigation;
import org.expeditee.gui.Frame;
import org.expeditee.gui.FrameIO;
import org.expeditee.gui.MessageBay;
import org.expeditee.items.ItemParentStateChangedEvent;
import org.expeditee.items.Picture;
import org.expeditee.items.Text;
import org.honours.agents.GenerateThumbnails;
import org.honours.collection.Collection;
import org.honours.gui.HonoursFrameIO;
import org.honours.items.PictureInformation;

public class CollectionSpace {
	
	//the collection this collection space is associated with
	private Collection _associatedCollection;
	
	private String _name;
	
	//FOr storing thumbnails for the collection space
	private String _thumbnailsDirectory;
	
	private List<Picture> _thumbnails = new ArrayList<Picture>();
	
	private int _startX = 150;
	private int _startY = 150;
	
	public static final int _gap = 20;
	public static final int _columns = 4;
	public static final int _imageSize = 150;
	public static final int _rows = 3;
	
	public CollectionSpace(){
		
	}
	
	public CollectionSpace(Collection associatedCollection){
		_associatedCollection = associatedCollection;
		_name = "collectionspace-" + _associatedCollection.getName();
		_thumbnailsDirectory = "expeditee" + File.separator + "thumbnails-" +
		_associatedCollection.getName();
	}
	
	public void setAssociatedCollection(Collection assocCollection){
		_associatedCollection = assocCollection;
	}
	
	public Collection getAssociatedCollection(){
		return _associatedCollection;
	}
	
	public void setName(String name){
		_name = name;
	}
	
	public String getName(){
		return _name;
	}
	
	public void setThumbnailsDirectory(String dir){
		
		_thumbnailsDirectory = dir;
		
	}
	
	public String getThumbnailsDirectory(){
		return _thumbnailsDirectory;
	}
	
	public void generate(){
		try{
			
		if(_thumbnailsDirectory == null){
			
			MessageBay.errorMessage("ERROR: No thumbnails directory set up so cannot generate CollectionSpace. Try running the GenerateThumbnails agent first before generating the Collection Space.");
			
			return;
		}

		if(!new File(_thumbnailsDirectory).exists()){
			GenerateThumbnails gt = new GenerateThumbnails();
			Frame toLoad = FrameIO.LoadFrame(_associatedCollection + "1");
			System.err.println("*** _associatedCollection = " + _associatedCollection);
			System.err.println("*** toLoad = " + toLoad);
			gt.process(FrameIO.LoadFrame(_associatedCollection.getName() + "1"));
		}
		
		//generate actual collectionSpace frameset using _name.
		createCollectionSpace();
		
		//Open thumbnails.inf file.
		BufferedReader br = new BufferedReader(new FileReader(
				_thumbnailsDirectory + File.separator +
				"thumbnails.inf"));
		String line;
		
		while((line = br.readLine()) != null){
			
			String[] split = line.split(",");
			
			//1st entry in split is filename, 2nd is associated frame name.
			Text imageSource = new Text("@i: " + split[0] + " 1.0");
			Picture newPic = PictureInformation.getPicture(imageSource);
			
			//Remove "thumbnail." from value in split[1].
			//String picLink = split[1].substring("thumbnail.".length());
			
			newPic.setLink(split[1]);
			//newPic.setLink(picLink);
			_thumbnails.add(newPic);
		}
		br.close();
		
		writeThumbnails();
		
		MessageBay.displayMessage("Collection Space generated!");
		
		Navigation.Goto(_name + "1");
		
		}catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
	/** Method to output thumbnails into collection space **/
	private void writeThumbnails(){
		
		Frame currFrame = null;
		int currX = _startX;
		int currY = _startY;
		
		int frameIter = 1;
		
		generateNavigationLinks(frameIter);
		generateDropBox(frameIter);
		
		for(int i = 0; i < _thumbnails.size(); i++)
		{
			Picture currPic = _thumbnails.get(i);
			
			if(i % _columns ==0 && i == 0){
				
			}else if(i % _columns == 0 && i > 0){
				currX = _startX;
				currY += _gap + _imageSize;
			}else{
				currX += _gap + _imageSize;
			}
			
			//Create a new frame to display next lot of items.
			if(i % (_columns * _rows) == 0 && i < _columns * _rows){
				currFrame = FrameIO.LoadFrame(_name + frameIter);
				
			}else if(i % (_columns * _rows) == 0 && i >= _columns * _rows){
				frameIter++;
				Frame check = FrameIO.LoadFrame(_name + frameIter);
				if(check == null){
					//create new frame
					HonoursFrameIO.generateFrame(frameIter, _name);
					currFrame = FrameIO.LoadFrame(_name + frameIter);
					
					currX = _startX;
					currY = _startY;
					generateNavigationLinks(frameIter);
					generateDropBox(frameIter);
				}

			}
			
			currFrame.addText(currX, currY, currPic.getSource().getText(),
					null, currPic.getLink());
			currFrame.setSaved();
		}
		
		_associatedCollection.setCollectionSpace(this);
	}
	
	private void generateNavigationLinks(int frameIter){
		
		String backText = "<<";
		String forwardText = ">>";
		
		int backButtonX = _startX/2;
		int forwardButtonX = _startX + (_columns*_gap)+(_columns*_imageSize);
		int buttonPosY = ((_rows*_imageSize) + (_rows*_gap))/2;
		
		Frame currFrame = FrameIO.LoadFrame(_name+frameIter);
		currFrame.addText(backButtonX,buttonPosY,backText,"RotateBack");
		currFrame.addText(forwardButtonX,buttonPosY,forwardText,"RotateNext");
		currFrame.setSaved();
	}
	
	private void generateDropBox(int frameIter){
		int maxRowLength = _startX + (_columns * _imageSize) + ((_columns-1)* _gap);
		
		//the maximum height of the area displaying all the rows.
		int maxRowsAreaHeight = _startY + (_rows * _imageSize) + ((_rows-1)*_gap);
		
		int dropBoxGap = 80;
		
		int dropBoxStartX = maxRowLength + (dropBoxGap*2);
		int dropBoxStartY = _startY;
		
		int dropBoxWidth = 300;
		int dropBoxHeight = maxRowsAreaHeight;
		
		Frame currFrame = FrameIO.LoadFrame(_name + frameIter);
		
		String dropBoxSourceText = "@iw: org.honours.widgets.CollectionSpaceDropBox " + dropBoxWidth + " " + dropBoxHeight;
		Text newText = currFrame.addText(dropBoxStartX, dropBoxStartY, dropBoxSourceText, null);
		newText.onParentStateChanged(new ItemParentStateChangedEvent(currFrame, ItemParentStateChangedEvent.EVENT_TYPE_ADDED));

		currFrame.setSaved();
	}
	
	private void createCollectionSpace(){
		Frame check = FrameIO.LoadFrame(_name + "1");
		
		if(check != null){
			FrameIO.deleteFrameset(_name);
		}
		
		try{
			FrameIO.CreateNewFrameset(_name);
		}catch(Exception e){
			e.printStackTrace();
		}
	}
}
